
# This file was automatically generated by Dist::Zilla::Plugin::ModuleBuild v6.008.
use strict;
use warnings;

use Module::Build 0.28;


my %module_build_args = (
  "build_requires" => {
    "Module::Build" => "0.28"
  },
  "configure_requires" => {
    "Module::Build" => "0.28"
  },
  "dist_abstract" => "a data toolkit",
  "dist_author" => [
    "Nicolas Steenlant, C<< <nicolas.steenlant at ugent.be> >>"
  ],
  "dist_name" => "Catmandu",
  "dist_version" => "1.0506",
  "license" => "perl",
  "module_name" => "Catmandu",
  "recommends" => {
    "Log::Any::Adapter::Log4perl" => "0.06",
    "Log::Log4perl" => "1.44"
  },
  "recursive_test_files" => 1,
  "requires" => {
    "Any::URI::Escape" => 0,
    "App::Cmd" => "0.33",
    "CGI::Expand" => "2.02",
    "Clone" => "0.31",
    "Config::Onion" => "1.004",
    "Cpanel::JSON::XS" => "3.0213",
    "Data::Compare" => "1.22",
    "Data::UUID" => "1.217",
    "File::Find::Rule" => "0.33",
    "Hash::Merge::Simple" => 0,
    "IO::Handle::Util" => "0.01",
    "LWP::UserAgent" => 0,
    "LWP::UserAgent::Determined" => 0,
    "List::MoreUtils" => "0.33",
    "Log::Any" => 0,
    "Log::Any::Adapter" => 0,
    "Module::Info" => 0,
    "Moo" => "1.004006",
    "MooX::Aliases" => "0.001006",
    "Parser::MGC" => "0.15",
    "Path::Tiny" => 0,
    "Ref::Util" => "0.020",
    "Sub::Exporter" => "0.982",
    "Sub::Quote" => 0,
    "Text::CSV" => "1.21",
    "Text::Hogan::Compiler" => "1.02",
    "Throwable" => "0.200004",
    "Time::HiRes" => 0,
    "Try::Tiny::ByClass" => "0.01",
    "URI" => 0,
    "URI::Template" => "0.22",
    "Unicode::Normalize" => 0,
    "YAML::XS" => "0.41",
    "asa" => 0,
    "namespace::clean" => "0.24",
    "perl" => "v5.10.1"
  },
  "script_files" => [
    "bin/catmandu"
  ],
  "test_requires" => {
    "Log::Any::Adapter::Log4perl" => 0,
    "Log::Any::Test" => "1.03",
    "Log::Log4perl" => 0,
    "Test::Deep" => "0.112",
    "Test::Exception" => "0.43",
    "Test::LWP::UserAgent" => 0,
    "Test::More" => "0.99",
    "Test::Pod" => 0
  }
);


my %fallback_build_requires = (
  "Log::Any::Adapter::Log4perl" => 0,
  "Log::Any::Test" => "1.03",
  "Log::Log4perl" => 0,
  "Module::Build" => "0.28",
  "Test::Deep" => "0.112",
  "Test::Exception" => "0.43",
  "Test::LWP::UserAgent" => 0,
  "Test::More" => "0.99",
  "Test::Pod" => 0
);


unless ( eval { Module::Build->VERSION(0.4004) } ) {
  delete $module_build_args{test_requires};
  $module_build_args{build_requires} = \%fallback_build_requires;
}

my $build = Module::Build->new(%module_build_args);


$build->create_build_script;
